package com.ningdatech.pmapi.user.manage;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.pmapi.common.util.StrUtils;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.sys.model.entity.Role;
import com.ningdatech.pmapi.sys.model.entity.UserRole;
import com.ningdatech.pmapi.sys.service.IRoleService;
import com.ningdatech.pmapi.sys.service.IUserRoleService;
import com.ningdatech.pmapi.user.convert.UserInfoConvertor;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.security.auth.credential.CredentialAuthToken;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.security.core.context.SecurityContextImpl;
import org.springframework.session.Session;
import org.springframework.session.SessionRepository;
import org.springframework.session.data.redis.RedisIndexedSessionRepository;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author liuxinxin
 * @date 2023/1/3 上午10:57
 */

@Component
@RequiredArgsConstructor
public class UserAuthLoginManage {

    private final IUserInfoService iUserInfoService;
    private final IRoleService iRoleService;
    private final IDingOrganizationService iDingOrganizationService;
    private final IDingEmployeeInfoService iDingEmployeeInfoService;
    private final IUserRoleService iUserRoleService;
    private final RedisIndexedSessionRepository redisIndexedSessionRepository;

    /**
     * 根据用户id 获取用户全量信息
     *
     * @param userId /
     * @return /
     */
    public UserFullInfoDTO getUserFullInfo(Long userId) {
        UserInfo userInfo = iUserInfoService.getById(userId);
        if (Objects.isNull(userInfo)) {
            return null;
        }
        // 返回用户全量信息
        UserFullInfoDTO userFullInfoDTO = new UserFullInfoDTO();
        userFullInfoAssembler(userInfo, userFullInfoDTO);
        return userFullInfoDTO;
    }

    /**
     * 根据用户名获取
     *
     * @param username /
     * @return /
     */
    public UserFullInfoDTO queryUserInfoInPasswordAuth(String username) {
        // TODO 目前账号密码登陆测试使用 强制用户登录 userId 1L
        UserInfo userInfo = iUserInfoService.getById(username);
        if (Objects.isNull(userInfo)) {
            return null;
        }
        // 返回用户全量信息
        UserFullInfoDTO userFullInfoDTO = new UserFullInfoDTO();
        userFullInfoAssembler(userInfo, userFullInfoDTO);
        return userFullInfoDTO;
    }

    /**
     * 根据手机号获取
     *
     * @param phoneNo /
     * @return /
     */
    public UserFullInfoDTO queryUserInfoInPhoneNoAuth(String phoneNo) {
        UserInfo userInfo = iUserInfoService.getOne(Wrappers.lambdaQuery(UserInfo.class)
                .eq(UserInfo::getMobile, phoneNo));
        if (Objects.isNull(userInfo)) {
            return null;
        }
        // 返回用户全量信息
        UserFullInfoDTO userFullInfoDTO = new UserFullInfoDTO();
        userFullInfoAssembler(userInfo, userFullInfoDTO);
        return userFullInfoDTO;
    }

    /**
     * 根据accountId(浙政钉扫码登陆)
     *
     * @param accountId /
     * @return /
     */
    public UserFullInfoDTO queryUserInfoInAccountIdAuth(String accountId) {

        // 获取用户信息
        UserInfo userInfo = iUserInfoService.getOne(Wrappers.lambdaQuery(UserInfo.class)
                .eq(UserInfo::getAccountId, accountId));
        if (Objects.isNull(userInfo)) {
            return null;
        }
        // 返回用户全量信息
        UserFullInfoDTO userFullInfoDTO = new UserFullInfoDTO();
        userFullInfoAssembler(userInfo, userFullInfoDTO);
        return userFullInfoDTO;
    }


    private void userFullInfoAssembler(UserInfo userInfo, UserFullInfoDTO userFullInfoDTO) {
        // 获取浙政钉雇员信息 组织信息
        String employeeCode = userInfo.getEmployeeCode();
        if (StringUtils.isNotBlank(employeeCode)) {
            List<DingEmployeeInfo> dingEmployeeInfoList = iDingEmployeeInfoService
                    .list(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                            .eq(DingEmployeeInfo::getEmployeeCode, employeeCode)
                            .eq(DingEmployeeInfo::getMainJob, "true"));
            DingEmployeeInfo dingEmployeeInfo = dingEmployeeInfoList.get(0);


            // 装配用户任职所在单位
            String empPosUnitCode = dingEmployeeInfo.getEmpPosUnitCode();
            if (StringUtils.isNotBlank(empPosUnitCode)) {
                DingOrganization dingOrganization = iDingOrganizationService.getByOrgCode(empPosUnitCode);
                if (Objects.nonNull(dingOrganization)) {
                    userFullInfoDTO.setEmpPosUnitCode(empPosUnitCode);
                    userFullInfoDTO.setEmpPosUnitName(dingOrganization.getOrganizationName());
                }
            }

            // 装配用户所在orgCode
            String organizationCode = dingEmployeeInfo.getOrganizationCode();
            List<DingOrganization> dingOrganizationList = iDingOrganizationService.list(Wrappers
                    .lambdaQuery(DingOrganization.class)
                    .eq(DingOrganization::getOrganizationCode, organizationCode));
            if (CollectionUtil.isNotEmpty(dingOrganizationList)) {
                DingOrganization dingOrganization = dingOrganizationList.get(0);
                userFullInfoDTO.setOrganizationCode(dingOrganization.getOrganizationCode());
                userFullInfoDTO.setOrganizationName(dingOrganization.getOrganizationName());
                userFullInfoDTO.setRegionCode(dingOrganization.getDivisionCode());
                // 测试使用
                userFullInfoDTO.setRegionLevel(3);
            }
        }

        List<Role> roleList = new ArrayList<>();
        // 获取用户角色列表信息
        List<UserRole> userRoleList = iUserRoleService
                .list(Wrappers.lambdaQuery(UserRole.class)
                        .eq(UserRole::getUserId, userInfo.getId()));
        if (CollectionUtil.isNotEmpty(userRoleList)) {
            List<Long> roleIdList = userRoleList.stream()
                    .map(UserRole::getRoleId).distinct()
                    .collect(Collectors.toList());
            roleList = iRoleService.list(Wrappers.lambdaQuery(Role.class).in(Role::getId, roleIdList));
        }
        userFullInfoDTO.setUserRoleList(roleList);

        // 装配用户任职所在单位

        // 装配返回
        userFullInfoDTO.setUserId(userInfo.getId());
        userFullInfoDTO.setIdentifier(userInfo.getRealName());
        userFullInfoDTO.setRealName(userInfo.getRealName());
        userFullInfoDTO.setEmployeeCode(employeeCode);
        userFullInfoDTO.setUsername(userInfo.getRealName());
        userFullInfoDTO.setMobile(userInfo.getMobile());
        userFullInfoDTO.setAccountId(userInfo.getAccountId());
    }

    public UserFullInfoDTO getUserFullInfoByEmployeeCode(String employeeCode) {
        UserInfo userInfo = iUserInfoService.getOne(Wrappers.lambdaQuery(UserInfo.class)
                .eq(UserInfo::getEmployeeCode, employeeCode).last("limit 1"));
        if (Objects.isNull(userInfo)) {
            return null;
        }
        // 返回用户全量信息
        UserFullInfoDTO userFullInfoDTO = new UserFullInfoDTO();
        userFullInfoAssembler(userInfo, userFullInfoDTO);
        return userFullInfoDTO;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    public void refreshSession(Long userId) {
        UserFullInfoDTO userFullInfo = getUserFullInfo(userId);
        List<String> sessionIds = getSessionIds(userFullInfo);
        if (sessionIds.isEmpty()) {
            return;
        }
        SessionRepository redisSessionRepository = redisIndexedSessionRepository;
        UserInfoDetails details = UserInfoConvertor.toUserInfoDetails(userFullInfo);
        CredentialAuthToken cat = new CredentialAuthToken(details, details.getPassword(), details.getAuthorities());
        sessionIds.forEach(sessionId -> {
            Session session = redisIndexedSessionRepository.findById(sessionId);
            SecurityContextImpl context = session.getAttribute("SPRING_SECURITY_CONTEXT");
            context.setAuthentication(cat);
            session.setAttribute("SPRING_SECURITY_CONTEXT", context);
            redisSessionRepository.save(session);
        });
    }

    private List<String> getSessionIds(UserFullInfoDTO userFullInfo) {
        List<String> sessionIds = new ArrayList<>();
        String identifier = userFullInfo.getIdentifier();
        if (StrUtils.isNotBlank(identifier)) {
            sessionIds.addAll(redisIndexedSessionRepository.findByPrincipalName(identifier).keySet());
        }
        String mobile = userFullInfo.getMobile();
        if (StrUtils.isNotBlank(mobile)) {
            sessionIds.addAll(redisIndexedSessionRepository.findByPrincipalName(mobile).keySet());
        }
        Long accountId = userFullInfo.getAccountId();
        if (accountId != null) {
            sessionIds.addAll(redisIndexedSessionRepository.findByPrincipalName(accountId.toString()).keySet());
        }
        return sessionIds;
    }

    public void kickOff(Long userId) {
        UserFullInfoDTO userFullInfo = getUserFullInfo(userId);
        List<String> sessionIds = getSessionIds(userFullInfo);
        sessionIds.forEach(redisIndexedSessionRepository::deleteById);
    }

}
